function nullVect(n) {
	var a = new Array(n);
	for(var i = a.length - 1; i>=0; i--) {
		a[i] = 0;
	}
	return a;
}

var Systeme = function(A, B) {
	this.mat = A;
	this.B = B;
	
	this.curL = 0;
	this.curC = 0;
	this.maxL = A.length - 1;
	this.maxC = A[0].length - 1;
	this.echelon = false;
	
	var isPivot = nullVect(A[0].length);
	
	this.echange = function(i, j) {
		var c;
		if (i !== j) {
			for(var k = this.maxC; k >= 0; k--) {
				//echange de chaque élément des lignes i et j dans A
				c = A[i][k];
				A[i][k] = A[j][k];
				A[j][k] = c;
			}
			//echange des éléments de B
			c = B[i];
			B[i] = B[j];
			B[j] = c;
			return true;
		}
		return false;
	};
	
	this.multiplie = function(i, lambda) {
		if (lambda.num != 0) {
			B[i].mult(lambda);
			for(var k = this.maxC; k >= 0; k--) {
				A[i][k].mult(lambda);
			}
			return true;
		}
		return false;
	};
	
	this.combinaison = function(i, j, lambda) {
		//Li <- Li + lambda Lj
		if (i !== j) {
			B[i].add(lambda.mult(B[j], true));
			for(var k = this.maxC; k >= 0; k--) {
				A[i][k].add(lambda.mult(A[j][k], true));
			}
			return true;
		}
		return false;
	};
	
	/**
	 * Cherche un coefficient non nul dans la colonne donnée, à partir de la ligne donnée.
	 * retourne l'indice de ligne du premier coeff non nul de cette colonne ou false.
	 */
	this.cheche_pivot_col = function(col, ligne) {
		var i = ligne;
		while(i <= this.maxL && A[i][col].isZero()) {
			i++;
		}
		if (i < A.length) {
			return i;
		}
		return false;
	};
	
	/**
	 * Cheche un coefficient non nul dans la ligne donné, à partir du numéro de colonne donné.
	 * Retourn l'indice de colonne ou false
	 */
	this.cheche_pivot_ligne = function(col, ligne) {
		var c = typeof col === "undefined" ? this.curC : col,
			l = typeof ligne === "undefined" ? this.curL : ligne;
		while(c <= this.maxC && A[l][c].isZero()) {
			c++;
		}
		if (c <= this.maxC) {
			return c;
		}
		return false;
	};


    this.construitPoint = function() {
        
    }
	
	/**
	 * Retourne les prochaines opérations à effectuer pour réduir le système.
	 * Retourne la liste vide si la système est réduit, et peuple this.sols dans ce cas.
	 */
	this.prochainesOps = function() {
		var ops = [];
		if (this.reduced) {
			return ops;
		}
		if (!this.echelon) {
			//1ère phase "descendante"
			if (this.curL < this.maxL && this.curC <= this.maxC) {
				var i = this.cheche_pivot_col(this.curC, this.curL);
				if (i === false) {
					this.curC++;
					return this.prochainesOps();
				}
				if (i !== this.curL) {//echange éventuel de ligne
					ops.push(new operations.Operation(operations.ECHANGE, [i, this.curL]));
					return ops;
				}
				//sinon, le pivot est dans la bonne position
				for(var li = i + 1; li <= this.maxL; li++) {
					if (!this.mat[li][this.curC].isZero()) {
						ops.push(new operations.Operation(operations.COMBINAISON,
							[li, i, this.mat[li][this.curC].div(this.mat[i][this.curC], true).opposite()]));
					}
				}
				this.curL++;
				this.curC++;
				return ops;
			} else {
				this.echelon = true;
				var comp = this.estCompatible();
				if (comp === false) {
					this.reduced = true;
					this.sols = new Solutions();
					return [];
				} else {
					this.curL = comp;
					this.rang = comp + 1;
				}
				return this.prochainesOps();
			}
		} else {
			if (this.curL >= 0) {
				var j = this.cheche_pivot_ligne(0, this.curL);
				//recherche du premier coefficient non nul de la ligne courante
				if (j === false) {
					this.curL--;
					return this.prochainesOps();
				}
				isPivot[j] = true;
				//console.log(isPivot, this.curL, j);
				//placer les pivots à 1 avant d'effectuer les opérations
				if (!this.mat[this.curL][j].isOne()) {
					return [new operations.Operation(operations.MULTIPLICATION, 
							[this.curL, this.mat[this.curL][j].inv(true)])];
				}
				
				for(var i = this.curL - 1; i >= 0; i--) {
					//if (!this.mat[i][j].isZero()) {
						ops.push(new operations.Operation(operations.COMBINAISON,
							[i, this.curL, this.mat[i][j].opposite(true)]));
					//}
				}
				this.curL--;
				return ops;
			} else {
				this.reduced = true;
				console.log(isPivot);
				//construction de l'ensemble des solutions.
				var paramNb = this.mat[0].length - this.rang,
					base,
					params = [],
					pivot,
					pivotIndice,
					point;
				if (paramNb === 0) {
					this.sols = new Solutions(this.B);
				} else {
					//construction de la base.
					base = new Array(paramNb);
					for(var i = paramNb - 1; i>=0; i--) {
						//chaque élément de la base possède nbInconnues coordonnées
						base[i] = nullVect(this.mat[0].length);
					}
					//params est la liste d'indice des paramètres
					for(i = 0; i < isPivot.length; i++) {
						if (!isPivot[i]) {
							params.push(i);
						}
					}
					if (params.length != paramNb) {
						alert('Incohérence dans le nombre de paramètre');
						console.log(params);
						return [];
					}
					
					//pour chaque ligne non nulle, aller chercher les coefficients des paramètres
					for(i = 0; i < this.rang; i++) {
						for(j = 0; j < params.length; j++) {
							base[j][i] = this.mat[i][params[j]].opposite(true);
						}
					}
					//remplir pour les paramètres;
					for(i = 0; i < paramNb; i++) {
						base[i][params[i]] = 1;
					}
					
					//construction du second membre
					
					point = Array.apply(null, Array(this.mat[0].length)).map(function() {return 0;});
					for(i = Math.min(point.length, this.B.length) - 1; i >=0; i--) {
						point[i] = this.B[i];
					}
					this.sols = new Solutions(point, base);
				}
				return ops;
			}
		}
	};
	
	/**
	 * Test if this system is compatible.
	 * Returns the rank or false in case of incompatible system
	 */
	this.estCompatible = function() {
		var i = this.maxL;
		while (i >=0 && this.cheche_pivot_ligne(0, i) === false) {
			if (this.B[i].isZero()) {
				i--;
			} else {
				return false;
			}
		}
		return i;
	};
	
},
Pivot = function(container) {
	this.A = [];
	this.B = [];
	this.container = container;
	this.sys = container.find('.sys');
	
	this.setSize = function(ligne, col) {
		var sys = this.sys,
			l, cel,
			//noms des inconnues par défaut.
			nomsIconnues = ['x', 'y', 'z', 't'];
		$('.systeme:not(:last)').remove();
		$('.ops').remove();
		sys.empty();
		this.A = new Array(ligne);
		this.B = new Array(ligne);
		if (col > 4) {//trop d'inconnues, passons aux indices
			for(var j = 0; j < col; j++) {
				nomsIconnues[j] = 'x<sub>' + (j + 1) + '</sub>';
			}
		}
		for (var i = 0; i < ligne; i++) {
			l = $('<tr>');
			this.A[i] = new Array(col);
			for (var j = 0; j < col; j++) {
				if (j !== 0) {
					l.append('<td class="plus">+</td>');
				}
				cel = $('<td class="coeff" contenteditable="true"></td>');
				cel.data('i', i).data('j', j);
				l.append(cel).append($('<td class="inconnue">')
						.html(nomsIconnues[j]));
				this.A[i][j] = cel;
			}
			l.append('<td class="egal">=</td>');
			cel = $('<td class="cste" contenteditable="true"></td>').data('i',i);
			this.B[i] = cel;
			l.append(cel);
			l.append('<td class="num-ligne">(L<sub>' + (i + 1) +'</sub>)</td>');
			sys.append(l);
		};
		
		this.setAccolade(container, sys);
	};
	
	//Place une accolade de la bonne taille à gauche du système donné.
	this.setAccolade = function(container, sys) {
		//set left {
		var acc = container.find('.accolade')[0];
		acc.height = sys.height();
		var ctx = acc.getContext('2d');
		ctx.clearRect(0, 0, acc.width, acc.height);
		ctx.textBaseline = 'bottom';
		ctx.textAlign = "center";
		ctx.font = (acc.height) + 'px sans-serif';
		ctx.fillText('{', 10, acc.height, 18);
	};
	
	this.init = function(ev) {
		if (ev) {
			ev.preventDefault();
			if (ev.originalEvent) {
				ev.originalEvent.preventDefault();
			}
			
		}
		
		ligne = parseInt($('#ligne').val(), 10);
		col = parseInt($('#col').val(), 10);
		
		if (!isNaN(ligne) && !isNaN(col)) {
			this.ligne = ligne;
			this.col = col;
			this.setSize(ligne, col);
		} else {
			this.setSize(0,0);
		}
		return false;
	};
	
	//Utilisé en interne pour initialiser les donnés par les champs d'entrée
	this._pop = function(creator) {
		var A = [],
			B = [];
		for(var i = this.A.length - 1; i >=0; i--) {
			A[i] = [];
			for(var j = this.A[0].length - 1; j >= 0; j--) {
				A[i][j] = creator.parse(this.A[i][j].text() || '0');
			}
			B[i] = creator.parse(this.B[i].text() || '0');
		}
		return [A, B];
	};
	
	this.populate = function(ev) {
		var l = [];
		try {
			l = this._pop(Fraction);
		} catch (e) {
			l = this._pop(Flottant);
		}
		this.systeme = new Systeme(l[0], l[1]);
	};
	
	//cré un nouveau système ainsi que la liste des opérations effectuées.
	this.updateView = function(ops) {
		var c = this.container.clone().removeAttr('id');
		this.container.before(c);
		for(var i = 0; i < ops.length; i++) {
			if (ops) {
				this.container.before($('<div class="ops">').html(ops[i].html()));
			}
		}
		this.setAccolade(c, c.find('.sys'));
		for(var i = this.A.length - 1; i>= 0; i--) {
			this.B[i].text(this.systeme.B[i]);
			for(var j = this.A[0].length - 1; j >= 0; j--) {
				this.A[i][j].text(this.systeme.mat[i][j]);
			}
		}
	};
	
	$('#set-size').on('submit',$.proxy(this.init, this));
	
	this.prochainesOps = function() {
		return this.systeme.prochainesOps();
	};
},
operations = {
	ECHANGE: 0,
	COMBINAISON: 1,
	MULTIPLICATION: 2,
	Operation: function(type, args) {
		this.type = type;
		this.args = args;
	}
},
Fraction = function(num, denom) {
	if (denom === 0) {
		throw new Error('Division par zéro');
	}
	
	this.num = num;
	this.denom = denom || 1;
	this.reduce();
},
Flottant = function(f) {
	this.f = f;
},
Solutions = function(A, base) {
	this.vide = typeof A === "undefined";
	
	this.toString = function() {
		if (this.vide) {
			return "Pas de solution.";
		} else {
			if (typeof base !== "undefined" && base.length > 0) {
				s = "L'ensemble des solutions est : ";
				s += '[' + A + ']';
				s += ' + Vect(';
				for(var i = 0; i < base.length; i++) {
					s += '[' + base[i] + ']';
					if (i < base.length - 1) {
						s += ', ';
					}
				}
				s += ')';
			} else {
				s = "La solution est : ";
				s += '[' + A + ']';
			}
		}
		return s;
	};
};

operations.Operation.prototype = {
	html : function() {
			var s = '';
			if (this.type == operations.ECHANGE) {
				s = 'L<sub>' + (this.args[0] + 1) + '</sub>';
				s += ' <-> ' + 'L<sub>' + (this.args[1] + 1) + '</sub>';
				return s;
			}
			if (this.type == operations.COMBINAISON) {
				s = 'L<sub>' + (this.args[0] + 1) + '</sub>';
				s += '<- ' + 'L<sub>' + (this.args[0] + 1) + '</sub> ';
				s += this.args[2].num >=0 ? '+ ': ' ';
				s += this.args[2];
				s += 'L<sub>' + (this.args[1] + 1) + '</sub>';
				return s;
			}
			if (this.type == operations.MULTIPLICATION) {
				s = 'L<sub>' + (this.args[0] + 1) + '</sub>';
				s += '<- ' + this.args[1] + 'L<sub>' + (this.args[0] + 1) + '</sub>';
				return s;
			}
		},
		
	applique : function(systeme) {
		if (this.type == operations.ECHANGE) {
			return systeme.echange(this.args[0], this.args[1]);
		}
		if (this.type == operations.COMBINAISON) {
			return systeme.combinaison(this.args[0], this.args[1], this.args[2]);
		}
		if (this.type == operations.MULTIPLICATION) {
			return systeme.multiplie(this.args[0], this.args[1]);
		}
	},
	toTex: function() {
		
	}
};

Fraction.pgcd = function(a, b) {
	var u = Math.abs(a),
		v = Math.abs(b),
		c;
	while(v !== 0) {
		c = u;
		u = v;
		v = c % v;
	}
	return u;
};

Fraction.parse = function(s) {
	var l = s.split('/'),
		num = NaN,
		denom = NaN;
	if (l.length >= 1 && Math.floor(l[0]) == l[0]) {
		num = parseInt(l[0], 10);
	}
	if (l.length === 2) {
		denom = parseInt(l[1], 10);
	} else {
		denom = 1;
	}
	if (!isNaN(num) && !isNaN(denom)) {
		return new Fraction(num, denom);
	} else {
		throw new Error('Chaîne invalide.');
	}
};

Flottant.parse = function(s) {
	var f = parseFloat(s);
	return new Flottant(f);
};

Fraction.prototype = {
	toString: function() {
		if (this.denom == 1) {
			return this.num;
		}
		return this.num + ' / ' + this.denom;
	},
	toFloat: function() {
		return this.num / this.denom;
	},
	add: function(f, create) {
		var num = this.num * f.denom + this.denom * f.num,
			denom = this.denom * f.denom;
		if (create) {
			return new Fraction(num, denom);
		}
		this.num = num;
		this.denom = denom;
		return this.reduce();
	},
	opposite: function(create) {
		var num = - this.num;
		if (create) {
			return new Fraction(num, this.denom);
		}
		this.num = num;
		return this;
	},
	sub: function(f, create) {
		var num = this.num * f.denom - this.denom * f.num,
			denom = this.denom * f.denom;
		if (create) {
			return new Fraction(num, denom);
		}
		this.num = num;
		this.denom = denom;
		return this.reduce();
	},
	mult: function(f, create) {
		var num = this.num * f.num,
			denom = this.denom * f.denom;
		if (create) {
			return new Fraction(num, denom);
		}
		this.num = num;
		this.denom = denom;
		return this.reduce();
	},
	scalMult: function(l, create) {
		var num = l * this.num;
		if (create) {
			return new Fraction(num, this.denom);
		}
		this.num = num;
		return this.reduce();
	},
	inv: function(create) {
		if (this.isZero()) {
			throw new Error('Division par 0');
		}
		var denom = this.num,
			num = this.denom;
		if (create) {
			return new Fraction(num, denom);
		}
		this.num = num;
		this.denom = denom;
		return this.reduce();
	},
	div: function(f, create) {
		if (f.isZero()) {
			throw new Error('Division par 0');
		}
		var denom = this.denom * f.num,
			num = this.num * f.denom;
		if (create) {
			return new Fraction(num, denom);
		}
		this.num = num;
		this.denom = denom;
		return this.reduce();
	},
	reduce: function() {
		var d = Fraction.pgcd(this.num, this.denom);
		this.num = this.num / d;
		this.denom = this.denom / d;
		
		if (this.denom < 0) {
			this.num = - this.num;
			this.denom = - this.denom;
		}
		return this;
	},
	isZero: function() {
		return this.num === 0;
	},
	isOne: function() {
		return this.num === this.denom;
	}
},
Flottant.prototype =  {
	tol: 1e-15,
	add: function(f, create) {
		var x = this.f + f.f;
		if (create) {
			return new Flottant(x);
		}
		this.f = x;
		return this;
	},
	sub: function(f, create) {
		var x = this.f - f.f;
		if (create) {
			return new Flottant(x);
		}
		this.f = x;
		return this;
	},
	opposite: function(create) {
		var x = -this.f;
		if (create) {
			return new Flottant(x);
		}
		this.f = x;
		return this;
	},
	mult: function(f, create) {
		var x = this.f * f.f;
		if (create) {
			return new Flottant(x);
		}
		this.f = x;
		return this;
	},
	div: function(f, create) {
		if (f.isZero()) {
			throw new Error('Division par 0.');
		}
		var x = this.f / f.f;
		if (create) {
			return new Flottant(x);
		}
		this.f = x;
		return this;
	},
	inv: function(create) {
		if (this.isZero()) {
			throw new Error('Division par 0.');
		}
		var x = 1 / this.f;
		if (create) {
			return new Flottant(x);
		}
		this.f = x;
		return this;
	},
	isZero: function() {
		return Math.abs(this.f) <= this.tol;
	},
	isOne: function() {
		return Math.abs(this.f - 1) <= this.tol;
	},
	toString: function() {
		return this.f.toString();
	}
};
